/*  tds200mt
 *  Copyright (C) 1999 Alvaro De Amicis
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include "tmt.h"
#include "comm.h"
#include "utils.h"

extern CommParam comm_param; 
static struct termios oldtrm;


int 
opentty ()
{
	int fd;
	struct termios trm;
	fd = open (comm_param.device, O_RDWR | O_NOCTTY);
	if (fd < 0){
		perror("open");
		return -1;
	}
	tcgetattr (fd, &oldtrm);	/* salva il setup attuale della porta */
	bzero (&trm, sizeof (trm));
	switch (comm_param.bps) {
	case TMT_B3:
		trm.c_cflag = B300 | CS8 | CREAD; break;
	case TMT_B6:
		trm.c_cflag = B600 | CS8 | CREAD; break;
	case TMT_B12:
		trm.c_cflag = B1200 | CS8 | CREAD; break;
	case TMT_B24:
		trm.c_cflag = B2400 | CS8 | CREAD; break;
	case TMT_B48:
		trm.c_cflag = B4800 | CS8 | CREAD; break;
	case TMT_B96:
		trm.c_cflag = B9600 | CS8 | CREAD; break;
	case TMT_B192:
		trm.c_cflag = B19200 | CS8 | CREAD; break;
	}
	switch (comm_param.flow) {
	case TMT_FRTSCTS:
		trm.c_cflag |= CRTSCTS ; break;
	case TMT_FXONXOFF:
		trm.c_iflag |= IXON | IXOFF ; break;
	}	
	trm.c_iflag |= IGNPAR;
	trm.c_oflag = 0;
	trm.c_lflag = 0;
	trm.c_cc[VTIME] = 0;
	trm.c_cc[VMIN] = 0;
	tcflush (fd, TCIFLUSH);
	tcsetattr (fd, TCSANOW, &trm);
	return fd;
}


int 
closetty (int fd)
{
	tcsetattr (fd, TCSANOW, &oldtrm);
	if (close (fd) < 0) {
		perror("close");
		return -1;
	}
	else
		return 0;
}


int 
send (int fd, char *msg, int len)
{
	if (!len) len = strlen (msg);
	if (write (fd, msg, len) == -1) {
		perror ("write");
		return -1;
	}
	return 0;
}


int 
rxwaveform (TdsWave *wf, Location src, int xsession)
{
	int fd, r, i, c;
	char buf[255];
	
	memset (wf->WFMdata, 0, 2500);
	memset (wf->WFMPreWFid, 0, 128);

	fd = opentty ();
	if (fd < 0) {
		if (xsession)
			show_message_box ("Error: cannot open tty\n");
		return -1;
	}

	send (fd, "VERBOS ON\r", 0);
	send (fd, "DATA:ENCDG RPB\r", 0);	/* unsigned          */
	send (fd, "DATA:WIDTH 1\r", 0);		/* 1 byte data width */
	send (fd, "DATA:START 1\r", 0);		/* first data point  */
	send (fd, "DATA:STOP 2500\r", 0);	/* last data point   */
	switch (src) {
	case TMT_CH1:
		send (fd, "DATA:SOURCE CH1\r", 0);
		break;
	case TMT_CH2:
		send (fd, "DATA:SOURCE CH2\r", 0);
		break;
	case TMT_MATH:
		send (fd, "DATA:SOURCE MATH\r", 0);
		break;
	case TMT_REFA:
		send (fd, "DATA:SOURCE REFA\r", 0);
		break;
	case TMT_REFB:
		send (fd, "DATA:SOURCE REFB\r", 0);
		break;
	}
	send (fd, "CURVE?\r", 0);

	c = 0;
	do {						/* butto via i primi 13 chars (":CURVE #42500") */
		r = read (fd, buf, 1);
		if (r == -1)
			perror ("read");
		else
			c += r;
	}
	while (c < 13);

	c = 0;
	do {						/* leggo i bytes della forma d'onda */
		r = read (fd, buf, 255);
		if (r == -1)
			perror ("read");
		if (r > 0) {
			for (i = 0; i < r; ++i)
				wf->WFMdata[c+i] = buf[i];
			c += r;
		}
		if (buf[r-1] == '\n')	/* ricevuto carattere End Of Line */
			break;
	}
	while (1);

	send (fd, "WFMPRE:WFID?\r", 0);

	c = 0;
	do {						/* butto via i primi 13 chars (":WFMPRE:WFID ") */
		r = read (fd, buf, 1);	
		if (r == -1)
			perror ("read");
		else
			c += r;
	}
	while (c < 13);

	c = 0;
	do {						/* leggo la stringa di descrizione */
								/* comprese le virgolette */
		r = read (fd, buf, 255);
		if (r == -1)
			perror ("read");
		if (r > 0) {
			for (i = 0; i < r; ++i)
				wf->WFMPreWFid[c+i] = buf[i];
			c += r;
		}
		if (buf[r-1] == '\n')	/* ricevuto carattere End Of Line */
			break;
	}
	while (1);

	closetty (fd);
	return 0;
}


int
txwaveform (TdsWave *wf, Location src)
{
	int fd;
	fd = opentty ();
	if (fd < 0) {
		show_message_box ("Error: cannot open tty\n");
		return -1;
	}
	send (fd, "VERBOS ON\r", 0);
	send (fd, "DATA:ENCDG RPB\r", 0);	/* unsigned          */
	send (fd, "DATA:WIDTH 1\r", 0);		/* 1 byte data width */
	send (fd, "DATA:START 1\r", 0);		/* first data point  */
	send (fd, "DATA:STOP 2500\r", 0);	/* last data point   */
	switch (src) {
	case TMT_REFA:
		send (fd, "DATA:DESTINATION REFA\r", 0);
		break;
	case TMT_REFB:
		send (fd, "DATA:DESTINATION REFB\r", 0);
		break;
	}
	send (fd, "CURVE #42500", 0);
	send (fd, wf->WFMdata, 2500);
	tcdrain (fd);
	closetty (fd);
	return 0;
}


int
rxsettings (char buf[])
{
	int fd, r, i, c;
	char b[255];
	
	memset (buf, 0, 2048);
	
	fd = opentty ();
	if (fd < 0) {
		show_message_box ("Error: cannot open tty\n");
		return -1;
	}

	send (fd, "VERBOS ON\r", 0);
	send (fd, "SET?\r", 0);

	c = 0;
	do {
		r = read (fd, b, 255);
		if (r == -1)
			perror ("read");
		if (r > 0) {
			for (i = 0; i < r; ++i)
				buf[c+i] = b[i];
			c += r;
		}
		if (b[r-1] == '\n')		/* ricevuto carattere End Of Line */
			break;
	}
	while (1);

	closetty (fd);
	return 0;
}


int
txsettings (char* buf)
{
	int fd;
	if (comm_param.bps > TMT_B24 ) {
		show_message_box ("Error: cannot transmit over 2400bps! \n");
		return -1;
	}
	fd = opentty ();
	if (fd < 0) {
		show_message_box ("Error: cannot open tty\n");
		return -1;
	}
	send (fd, buf, 0);
	tcdrain (fd);
	delay(2);
	closetty (fd);
	return 0;
}

